part of bolter;

class _BolterBuilderElement<T> extends StatelessElement {
  final T Function() getter;
  _BolterBuilderElement(super.widget, this.getter);

  late final _bolter = bolter;

  @override
  bool get debugDoingBuild => false;

  @override
  void deactivate() {
    _bolter.stopListen(_notification);
    super.deactivate();
  }

  @override
  void mount(Element? parent, Object? newSlot) {
    super.mount(parent, newSlot);
    _bolter.listen(getter, _notification);
  }

  void _notification() {
    markNeedsBuild();
  }
}

abstract class BolterBuilder<T> extends StatelessWidget {
  factory BolterBuilder({
    Key? key,
    required final T Function() getter,
    required final Widget Function(BuildContext context, T value) builder,
  }) =>
      BolterBuilder(key: key, getter: getter, builder: builder);

  static withPresenter<T, P>({
    Key? key,
    required final T Function() getter,
    required final Widget Function(BuildContext context, P presenter, T value)
        builder,
  }) =>
      BolterBuilderWithPresenter(key: key, getter: getter, builder: builder);
}

class BolterBuilderRegular<T> extends StatelessWidget {
  final T Function() getter;
  final Widget Function(BuildContext context, T value) builder;

  const BolterBuilderRegular({
    super.key,
    required this.getter,
    required this.builder,
  });

  @override
  StatelessElement createElement() {
    return _BolterBuilderElement(this, getter);
  }

  @override
  Widget build(BuildContext context) {
    return builder(context, getter());
  }
}

class BolterBuilderWithPresenter<T, P extends Presenter<P>>
    extends StatelessWidget {
  final T Function() getter;
  final Widget Function(BuildContext context, P presenter, T value) builder;

  const BolterBuilderWithPresenter({
    super.key,
    required this.getter,
    required this.builder,
  });

  @override
  StatelessElement createElement() {
    return _BolterBuilderElement(this, getter);
  }

  @override
  Widget build(BuildContext context) {
    return builder(context, context.presenter<P>(), getter());
  }
}
